# coding: utf-8
from django.db import models

from representatives.models import Representative, TimeStampedModel


class Dossier(TimeStampedModel):
    title = models.CharField(max_length=1000)
    reference = models.CharField(max_length=200, unique=True)
    text = models.TextField(blank=True, default='')
    link = models.URLField()
    ext_link = models.URLField(blank=True, default='')

    class Meta:
        unique_together = (('title', 'reference'))

    def __unicode__(self):
        return unicode(self.title)


class Proposal(TimeStampedModel):
    dossier = models.ForeignKey(Dossier, related_name='proposals')
    title = models.CharField(max_length=1000, unique=True)
    description = models.TextField(blank=True, default='')
    reference = models.CharField(max_length=200, blank=True, null=True)
    datetime = models.DateTimeField(db_index=True)
    kind = models.CharField(max_length=200, blank=True, default='')
    total_abstain = models.IntegerField()
    total_against = models.IntegerField()
    total_for = models.IntegerField()

    representatives = models.ManyToManyField(
        Representative, through='Vote', related_name='proposals'
    )

    class Meta:
        ordering = ['datetime']
        unique_together = (('dossier', 'title', 'reference',
                            'kind', 'total_abstain', 'total_against',
                            'total_for'))

    @property
    def status(self):
        if self.total_for > self.total_against:
            return 'adopted'
        else:
            return 'rejected'

    def __unicode__(self):
        return unicode(self.title)


class Vote(models.Model):
    VOTECHOICES = (
        ('abstain', 'abstain'),
        ('for', 'for'),
        ('against', 'against')
    )

    proposal = models.ForeignKey(Proposal, related_name='votes')

    representative = models.ForeignKey(
        Representative, related_name='votes', null=True)
    # Save representative name in case of we don't find the representative
    representative_name = models.CharField(max_length=200, blank=True)

    position = models.CharField(max_length=10, choices=VOTECHOICES)

    class Meta:
        ordering = ['proposal__datetime']
        unique_together = (('proposal', 'representative'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0010_proposal_datetime_index'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='dossier',
            unique_together=set([('title', 'reference')]),
        ),
        migrations.AlterUniqueTogether(
            name='proposal',
            unique_together=set([('dossier', 'title', 'reference', 'kind', 'total_abstain', 'total_against', 'total_for')]),
        ),
        migrations.RemoveField(
            model_name='dossier',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='proposal',
            name='fingerprint',
        ),
    ]



# coding: utf-8

import sys
import ijson
import logging

import django
from django.apps import apps

from representatives_votes.models import Dossier

logger = logging.getLogger(__name__)


def find_dossier(data):
    '''
    Find dossier with reference matching either 'url_an' or 'url_sen',
    create it if not found.  Ensure its reference and source are 'url_an' if
    both fields are present.
    '''

    changed = False
    dossier = None
    reffield = None

    for field in [k for k in ('url_an', 'url_sen') if k in data]:
        try:
            dossier = Dossier.objects.get(reference=data[field])
            reffield = field
            break
        except Dossier.DoesNotExist:
            pass

    if dossier is None:
        reffield = 'url_an' if 'url_an' in data else 'url_sen'
        dossier = Dossier(reference=data[reffield])
        logger.debug('Created dossier %s' % data[reffield])
        changed = True

    if 'url_an' in data and reffield != 'url_an':
        logger.debug('Changed dossier reference to %s' % data['url_an'])
        dossier.reference = data['url_an']
        changed = True

    return dossier, changed


def parse_dossier_data(data):
    dossier, changed = find_dossier(data)

    thisurl = data['url_an' if data['chambre'] == 'AN' else 'url_sen']

    if dossier.reference != dossier.link:
        logger.debug('Changed dossier link to %s' % dossier.reference)
        dossier.link = dossier.reference
        changed = True

    title = data['titre']
    if dossier.reference == thisurl and dossier.title != title:
        logger.debug('Changed dossier title to %s' % title)
        dossier.title = title
        changed = True

    if 'url_an' in data and 'url_sen' in data:
        ext_link = data['url_sen']
        if dossier.ext_link != ext_link:
            logger.debug('Changed dossier ext. link to %s' % ext_link)
            dossier.ext_link = ext_link
            changed = True

    if changed:
        logger.debug('Saved dossier %s' % dossier.reference)
        dossier.save()


def main(stream=None):
    if not apps.ready:
        django.setup()

    for data in ijson.items(stream or sys.stdin, 'item'):
        parse_dossier_data(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0006_duplicates'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='vote',
            unique_together=set([('proposal', 'representative')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0008_unique_proposal_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='dossier',
            name='ext_link',
            field=models.URLField(default=b'', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0003_auto_20150702_1827'),
        ('representatives_votes', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='vote',
            name='representative_remote_id',
        ),
        migrations.AddField(
            model_name='vote',
            name='representative',
            field=models.ForeignKey(
                related_name='votes',
                to='representatives.Representative',
                null=True),
        ),
        migrations.AddField(
            model_name='vote',
            name='representative_fingerprint',
            field=models.CharField(
                max_length=200,
                blank=True),
        ),
        migrations.AlterField(
            model_name='vote',
            name='representative_name',
            field=models.CharField(
                default='',
                max_length=200,
                blank=True),
            preserve_default=False,
        ),
    ]



from django import test

from responsediff.response import Response


class RepresentativeManagerTest(test.TestCase):
    fixtures = ['representatives_votes_test.json']

    def functional_test(self, queries, url):
        with self.assertNumQueries(queries):
            result = test.client.Client().get(
                url,
                HTTP_ACCEPT='application/json; indent=4',
            )
        Response.for_test(self).assertNoDiff(result)

    def test_dossier(self):
        # One for dossier + 1 for proposals
        self.functional_test(2, '/api/dossiers/1/')

    def test_dossiers(self):
        self.functional_test(1, '/api/dossiers/')

    def test_proposal(self):
        # One for proposal and dossier + 1 for votes
        self.functional_test(2, '/api/proposals/1/')

    def test_proposals(self):
        self.functional_test(1, '/api/proposals/')

    def test_vote(self):
        self.functional_test(1, '/api/votes/1/')

    def test_votes(self):
        self.functional_test(1, '/api/votes/')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0004_auto_20150709_0819'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dossier',
            name='reference',
            field=models.CharField(unique=True, max_length=200),
        ),
    ]



from setuptools import find_packages, setup

setup(
    name='django-representatives-votes',
    version='0.0.19',
    description='Base app for government representative votes',
    author='Olivier Le Thanh Duong, Laurent Peuch, Arnaud Fabre, James Pic, Nicolas Joyard',
    author_email='olivier@lethanh.be',
    url='http://github.com/political-memory/django-representatives-votes',
    packages=find_packages(),
    include_package_data=True,
    license='GPLv3',
    keywords='django government parliament votes',
    install_requires=[
        'django>1.8,<1.9',
        'django-representatives>=0.0.27',
        'py-dateutil>=2,<3',
        'ijson>=2,<3',
        'pytz',  # Always use up-to-date TZ data
    ],
    extras_require={
        'api': [
            'django-filter>=0.13,<0.14',
            'djangorestframework>=3,<4',
        ],
        'testing': [
            'codecov>=2,<3',
            'flake8>=2,<3',
            'mock>=2,<3',
            'pep8>=1,<2',
            'pytest>=2,<3',
            'pytest-django>=2,<3',
            'pytest-cov>=2,<3',
            'django-responsediff>=0.6,<0.7'
        ]
    },
    entry_points={
        'console_scripts': [
            'parltrack_import_dossiers = representatives_votes.contrib.parltrack.import_dossiers:main',
            'parltrack_import_votes = representatives_votes.contrib.parltrack.import_votes:main',
            'francedata_import_dossiers = representatives_votes.contrib.francedata.import_dossiers:main',
            'francedata_import_scrutins = representatives_votes.contrib.francedata.import_scrutins:main',
            'francedata_import_votes = representatives_votes.contrib.francedata.import_votes:main',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
